@@ -56,7 +56,7 @@ class Auth(object):
 
     return Blueprint(
       'auth',
-      project.config['PROJECT']['APP_FOLDER'] + '.auth',
+      project.config['PROJECT']['FLASK_ROOT_FOLDER'] + '.auth',
       template_folder=abspath(join(dirname(__file__), 'templates', 'auth')),
       url_prefix=self.config['URL_PREFIX']
     )
@@ -86,12 +86,12 @@ class Auth(object):
 
     @project.before_startup
     def handler(project):
-      project.app.register_blueprint(self.blueprint)
-      self.login_manager.setup_app(project.app)
+      project.flask.register_blueprint(self.blueprint)
+      self.login_manager.setup_app(project.flask)
 
       if self.config['PROTECT_ALL_VIEWS']:
 
-        @project.app.before_request
+        @project.flask.before_request
         def check_if_logged_in():
           if (request.blueprint != 'auth'
               and request.endpoint # favicon
@@ -123,9 +123,9 @@ class GoogleAuth(Auth):
     'ACCESS_TYPE': "offline",
   }
 
-  def _before_register(self, project):
+  def on_register(self, project):
 
-    super(GoogleAuth, self)._before_register(project)
+    super(GoogleAuth, self).on_register(project)
 
     emails = self.config['AUTHORIZED_EMAILS']
     if isinstance(emails, list):
